#!/usr/bin/python

# Reflection Rejection
# by Nirav Patel
#
# Released into the public domain on 2009/02/06.
#
# Requires linux, SDL, and a v4l2 supported webcam.
# Use the arrow keys and the world around you to play.

import time
import pygame
import pygame.camera
from pygame.locals import *

pygame.init()

class ReflectionGame(object):
   size = ( 640, 480 )
   def __init__(self, **argd):
       self.__dict__.update(**argd)
       super(ReflectionGame, self).__init__(**argd)
       self.display = pygame.display.set_mode( self.size )
       pygame.display.set_caption('Reflection Rejection')
       cameralist = pygame.camera.list_cameras()
       self.camera = pygame.camera.Camera(cameralist[0], self.size, "HSV")
       self.camera.start()
       self.camera.set_controls(hflip=1)
       self.clock = pygame.time.Clock()
       self.snapshot = pygame.surface.Surface(self.size, 0, self.display)
       self.lines = pygame.surface.Surface(self.size, 0, self.display)
       self.reflection = pygame.surface.Surface(self.size, 0, self.display)
       self.bigheart = pygame.image.load('heart.png')
       self.bigheart = self.bigheart.convert()
       self.bigheart.set_colorkey((255,255,255))
       self.heart = pygame.transform.scale(self.bigheart, (20,20))
       self.heartmask = pygame.mask.from_surface(self.heart)
       self.reflection.set_colorkey((0,0,0))
       self.rmask = pygame.mask.Mask((640,480))
       self.ccolor = None
       self.starty = 240
       self.startx = 30
       self.y = self.starty
       self.x = self.startx
       self.yv = 0
       self.xv = 0
       self.ya = 0
       self.xa = 0
       self.heartsize = 10

      
   def get_and_flip(self):
       if self.camera.query_image():
           self.snapshot = self.camera.get_image(self.snapshot)
           self.ccolor = pygame.transform.average_color(self.snapshot)
           mask2 = pygame.mask.from_threshold(self.snapshot, self.ccolor, (40,100,100))
           self.reflection.fill((0,0,0))
           mask2.invert()
           ccl = mask2.connected_components(200)
           self.rmask.clear()
           for cc in ccl:
             self.rmask.draw(cc,(0,0))
             plist = cc.outline(20)
             if len(plist) > 2:
                 ccc = cc.centroid()
                 pygame.draw.polygon(self.reflection, (ccc[0]*255/640,ccc[1]*255/480,150), plist)
       

   def move_player(self):
       self.yv += 0.5 + self.ya
       self.xv += self.xa
       self.yv *= 0.9
       self.xv *= 0.9
       self.lasty = self.y
       self.lastx = self.x
       self.y += int(self.yv)
       self.x += int(self.xv)
       pygame.draw.line(self.lines,(200,150,150),(self.lastx,self.lasty),(self.x,self.y),1)
       overlap = self.rmask.overlap(self.heartmask,(self.x-self.heartsize,self.y-self.heartsize))
       if self.x < 0 or self.y < 0 or self.y > 480 or (overlap and self.x > 100):
          self.restart_level()
       elif self.x > 640:
          self.next_level()

   def composite(self):
       self.display.fill((0,0,0))
       self.display.blit(self.lines,(0,0))
       self.reflection.fill((0,0,0),(0,0,100,480))
       self.display.blit(self.reflection,(0,0))
       self.display.blit(self.heart,(self.x-self.heartsize,self.y-self.heartsize))
       pygame.display.flip()
       
   def restart_level(self):
       soutline = self.heart.copy()
       soutline.fill((0,0,0))
       olist = self.heartmask.outline()
       pygame.draw.lines(soutline,(200,150,150),1,olist)
       self.lines.blit(soutline,(self.x-self.heartsize,self.y-self.heartsize))
       self.yv = self.xv = 0
       self.y = self.starty
       self.x = self.startx
       
   def next_level(self):
       self.heartsize *= 1.5
       self.heartsize = int(self.heartsize)
       self.heart = pygame.transform.scale(self.bigheart,(self.heartsize*2,self.heartsize*2))
       self.heartmask = pygame.mask.from_surface(self.heart)
       self.yv = self.xv = 0
       self.y = self.starty
       self.x = self.startx

   def main(self):
       going = True
       while going:
           events = pygame.event.get()
           for e in events:
               if e.type == QUIT or (e.type == KEYDOWN and e.key == K_ESCAPE):
                   going = False
               elif e.type == KEYDOWN:
                   if e.key == K_UP:
                      # activate two player mode
                      self.ya = -1
                   elif e.key == K_DOWN:
                      self.ya = 1
                   elif e.key == K_LEFT:
                      self.xa = -1
                   elif e.key == K_RIGHT:
                      self.xa = 1
               elif e.type == KEYUP:
                  if e.key == K_UP and self.ya == -1:
                      self.ya = 0
                  if e.key == K_DOWN and self.ya == 1:
                      self.ya = 0
                  if e.key == K_LEFT and self.xa == -1:
                      self.xa = 0
                  if e.key == K_RIGHT and self.xa == 1:
                      self.xa = 0
           self.get_and_flip()
           self.move_player()
           self.composite()
           self.clock.tick(30)
       self.camera.stop()
ReflectionGame().main()
